/**
 ** mhpManipulationCodels.cc
 **
 ** Codels called by execution task mhpMainTask
 **
 ** Author: E. Akin Sisbot
 ** Date: Nov 06, update Aug 09
 **
 **/

#include <portLib.h>

#include "server/mhpHeader.h"
#include "mhpm3d.h"

hri_gik * MHP_GIK = NULL;
gik_task_joints * GIK_TASKS = NULL;

static POSTER_ID posterArmPoseOnTraj = NULL;


#if defined(JIDO) || defined (PR2)
static ManipulationPlanner * MANIP_PLANNER = NULL;
#endif

#if defined(USE_MIGHTABILITY_MAPS)
extern int SHOW_CURRENT_TASK_CANDIDATE_POINTS;
extern int SHOW_WEIGHT_BY_COLOR_FOR_CANDIDATE_POINTS;
extern p3d_env *envPt_MM;
extern std::vector<HRI_task_node> HRI_task_list;
extern int IS_PERFORMING_AGENT_MASTER;
std::vector <SM_TRAJ> curr_smTrajs;
SM_TRAJ_STR curr_SM_traj_for_poster;
extern int HRI_TASK_PLAN_IN_CARTESIAN;
extern int CONSIDER_OBJECT_DIMENSION_FOR_CANDIDATE_PTS;
extern int SHOW_CURRENT_TASK_CANDIDATE_POINTS;
extern std::map<int,std::string > HRI_task_NAME_ID_map;
extern char SUPPORT_NAME_FOR_HUMAN_TO_PUT_OBJ[50];

#endif

int SM_TRAJ_ID = 0;

ACTIVITY_EVENT mhp_initialize_manipulation(int *report)
{
  
  MHP_GIK = hri_gik_create_gik();
  
  if(MHP_GIK == NULL){
    printf("**MHP** Problem occured in the creation of GIK\n");
    *report =  S_mhp_GIK_NOT_CREATED;
    return FAIL;
  }
  
  GIK_TASKS = mhp_create_defaut_gik_tasks();
  if(GIK_TASKS == NULL){
    printf("**MHP** Problem occured in the creation of default motions. Wrong robot choice?\n");
    *report =  S_mhp_GIK_NOT_CREATED;
    return FAIL;
  }
  
  if (MHP_BTSET !=NULL && MHP_BTSET->human !=NULL && MHP_BTSET->robot != NULL){
    MHP_GIK->robot = MHP_BTSET->robot;
    MHP_GIK->human = MHP_BTSET->human;
    MHP_GIK->human_no = MHP_BTSET->human_no;
  }

#if defined(JIDO) || defined (PR2)
  if (MANIP_PLANNER == NULL) {
    p3d_rob * robotPt= MHP_ROBOT;
    if(robotPt==NULL)
      printf("cannot fetch the robot by its name\n");
    else {
      printf("initialize manip_planner\n");
      MANIP_PLANNER= new ManipulationPlanner(robotPt);
      printf("initialize manip_planner   OK\n");
    }
  }
#endif
  return ETHER;
}

/* mhpDoTaskWithAgentMain  -  codel EXEC of DoTaskWithAgent
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
mhpDoTaskWithAgentMain(MHP_AGENT_MOTION_TASK *motionTask, MHP_AGENT_MOTION_TASK_RESULT *motionTaskResult, int *report)
{
  int i,j;
  HRI_AGENT *agent = NULL;
  p3d_rob *object = NULL;
  configPt q, qs;
  p3d_vector3 Tcoord[3];
 
  if(!mhp_fetch_and_update_env(report))
    printf("Warning: Cannot fetch the current environment\n");
  
  for(i=0; i<MHP_AGENTS->all_agents_no; i++) {
    if( strcasestr(MHP_AGENTS->all_agents[i]->robotPt->name, motionTask->name.name) ) {
      agent = MHP_AGENTS->all_agents[i];
      break;
    }
  }
  if(agent == NULL){
    *report = S_mhp_AGENT_NOT_FOUND;
    return ETHER;
  }
  for(i=0; i<MHP_ENV->nr; i++){
    if( strcasestr(MHP_ENV->robot[i]->name, motionTask->task.target.name) ){
      object = MHP_ENV->robot[i];
      break;
    }
  }
  if(object == NULL){
    *report = S_mhp_OBJECT_UNKNOWN;
    return ETHER;
  }
  
  qs = MY_ALLOC(double, agent->robotPt->nb_dof); /* ALLOC */
  q  = MY_ALLOC(double, agent->robotPt->nb_dof); /* ALLOC */
  
  p3d_get_robot_config_into(agent->robotPt, &qs);
  
  Tcoord[0][0] = Tcoord[1][0] = Tcoord[2][0] = object->joints[1]->abs_pos[0][3];      
  Tcoord[0][1] = Tcoord[1][1] = Tcoord[2][1] = object->joints[1]->abs_pos[1][3];     
  Tcoord[0][2] = Tcoord[1][2] = Tcoord[2][2] = object->joints[1]->abs_pos[2][3];
  
  if(motionTask->task.motiontype == MHP_LOOK){
    hri_agent_single_task_manip_move(agent, GIK_LOOK, object, &q);
    p3d_set_and_update_this_robot_conf(agent->robotPt,q);
    for(i=0, j=0; i<agent->robotPt->nb_dof && j<4; i++) {
      if(q[i] != qs[i]) {
	motionTaskResult->look[j] = RTOD(q[i]);
	j++;
      }
    }
    motionTaskResult->look_nb = j-1;
  }
  if(motionTask->task.motiontype ==  MHP_LEFT_HAND_POINT){
    hri_agent_single_task_manip_move(agent, GIK_LAPOINT, object, &q);
    p3d_set_and_update_this_robot_conf(agent->robotPt,q);
    for(i=0, j=0; i<agent->robotPt->nb_dof && j<7; i++) {
      if(q[i] != qs[i]) {
	motionTaskResult->arm[j] = RTOD(q[i]);
	j++;
      }
    }
    motionTaskResult->look_nb = j-1;
  }
  if(motionTask->task.motiontype ==  MHP_RIGHT_HAND_POINT){
    hri_agent_single_task_manip_move(agent, GIK_RAPOINT, object, &q);
    p3d_set_and_update_this_robot_conf(agent->robotPt,q);
    for(i=0, j=0; i<agent->robotPt->nb_dof && j<7; i++) {
      if(q[i] != qs[i]) {
	motionTaskResult->arm[j] = RTOD(q[i]);
	j++;
      }
    }
    motionTaskResult->look_nb = j-1;
  }
  if(motionTask->task.motiontype ==  MHP_LEFT_HAND_REACH){
    hri_agent_single_task_manip_move(agent, GIK_LAREACH, object, &q);
    p3d_set_and_update_this_robot_conf(agent->robotPt,q);
    for(i=0, j=0; i<agent->robotPt->nb_dof && j<7; i++) {
      if(q[i] != qs[i]) {
	motionTaskResult->arm[j] = RTOD(q[i]);
	j++;
      }
    }
    motionTaskResult->look_nb = j-1;
  }
  if(motionTask->task.motiontype ==  MHP_RIGHT_HAND_REACH){
    hri_agent_single_task_manip_move(agent, GIK_RAREACH, object, &q);
    p3d_set_and_update_this_robot_conf(agent->robotPt,q);
    for(i=0, j=0; i<agent->robotPt->nb_dof && j<7; i++) {
      if(q[i] != qs[i]) {
	motionTaskResult->arm[j] = RTOD(q[i]);
	j++;
      }
    }
    motionTaskResult->look_nb = j-1;
  }
  
  MY_FREE(q, double , agent->robotPt->nb_dof); /* FREE */
  MY_FREE(qs, double, agent->robotPt->nb_dof); /* FREE */
  
  return ETHER;
}


/* mhpPlanPathStart  -  codel START of PlanPath
   Returns:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
mhpPlanPathStart(MHP_MOTION_TARGET *target, int *report)
{
  if(MHP_GIK == NULL){
    printf("**MHP** GIK components are not created. Cannot plan a path\n");
    *report =  S_mhp_NOT_INITIALIZED;
    return ETHER;
  }
  else{
    return EXEC;
  }
}


/* mhpPlanPathMain  -  codel EXEC of PlanPath
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
mhpPlanPathMain(MHP_MOTION_TARGET *target, int *report)
{
  int i;
  int Pathpointindex = 0;
  double Tcoord[6];
  gsl_vector * DT;
  configPt robotq;
  double begpos[6];
  double curpos[6];
  double  remainingdist = 100;
  int max_count = 0;
  double  maxdistance = 0;

  if(!mhp_fetch_and_update_env(report))
    printf("Warning: Cannot fetch the current environment\n");

  if(!strcmp(target->target.name,"COORD")){
    Tcoord[0] = target->coords.x; Tcoord[1] = target->coords.y;  Tcoord[2] = target->coords.z;
    Tcoord[3] = target->coords.yaw; Tcoord[4] = target->coords.pitch; Tcoord[5] = target->coords.roll;
  }
  else{
    if(MHP_ENV == NULL){
      printf("**MHP** MHP not correctly initialized.\n ");
      *report = S_mhp_NOT_INITIALIZED;
      return FAIL;
    }
    for(i=0; i<MHP_ENV->nr; i++){
      if( !strcmp(target->target.name,MHP_ENV->robot[i]->name) ){
	p3d_get_robot_pos(MHP_ENV->robot[i],Tcoord);
	break;
      }
    }
    if(i==MHP_ENV->nr){
      printf("Object not found. Cannot plan a path\n");
      *report = S_mhp_OBJECT_UNKNOWN;
      return FAIL;
    }
    else{
      printf("Planning path to %s\n",MHP_ENV->robot[i]->name);
    }
  }
  printf("MHP plans to reach the target: %f %f %f %f %f %f\n",Tcoord[0],Tcoord[1],Tcoord[2],Tcoord[3],Tcoord[4],Tcoord[5]);

  /* ----Beginning GIK task----- */

  /* TODO: It might be good to do simple distance tests to see if the robot can reach the goal */

 
  /* initialize gik */
  
  if( MHP_GIK == NULL || MHP_ROBOT == NULL){
    printf("**MHP** MHP not correctly initialized.\n ");
    *report = S_mhp_NOT_INITIALIZED;
    return FAIL;
  }
  if(MHP_GIK->GIKInitialized){
    hri_gik_destroy_gik_data(MHP_GIK);		
    hri_gik_uninitialize_gik(MHP_GIK);	
  }

  DEBUG_PRINTF(3,("GIK Initialization\n"));
  hri_gik_initialize_gik(MHP_GIK,MHP_ROBOT,GIK_TASKS[target->motiontype].actual_joints_no);
  
  DEBUG_PRINTF(3,("GIK Add Task\n"));
  hri_gik_add_task(MHP_GIK, 3, GIK_TASKS[target->motiontype].actual_joints_no, 1, 
		   GIK_TASKS[target->motiontype].actual_joints,GIK_TASKS[target->motiontype].active_joint);

  /* -----BEGIN PATH PLANNING----- */
  DEBUG_PRINTF(3,("GIK beginning\n"));

  
  for(i=0; i<MHP_GIK->task_no; i++){ 
    gsl_vector_set(MHP_GIK->task[i]->goal, 0, Tcoord[0]);
    gsl_vector_set(MHP_GIK->task[i]->goal, 1, Tcoord[1]); 
    gsl_vector_set(MHP_GIK->task[i]->goal, 2, Tcoord[2]); /* We take into account only the position */
  }
  
  SDI_F->State.planning = TRUE; /* --- Planning Mode ---- */

  robotq = MY_ALLOC(double,MHP_GIK->robot->nb_dof); /* ALLOC */
  p3d_get_robot_config_into(MHP_GIK->robot,&robotq);

  DT = gsl_vector_alloc(MHP_GIK->joint_no); /* ALLOC */

  p3d_mat4ExtractPosReverseOrder(MHP_GIK->robot->joints[GIK_TASKS[target->motiontype].active_joint]->abs_pos, begpos, begpos+1, begpos+2,begpos+3, begpos+4, begpos+5);
  
  do{
    
    for(i=0; i<MHP_GIK->task_no; i++)
      hri_gik_computeJacobian(MHP_GIK,i,0);    
    
    hri_gik_free_joints(MHP_GIK);    
    
    hri_gik_compute_core(MHP_GIK, DT);
    
    hri_gik_updaterobot(MHP_GIK, DT);

    /* here the robot is moved a little towards the goal */

    p3d_mat4ExtractPosReverseOrder(MHP_GIK->robot->joints[GIK_TASKS[target->motiontype].active_joint]->abs_pos, 
				   curpos, curpos+1, curpos+2, curpos+3, curpos+4, curpos+5);
    
    if(MHP_D3D(begpos[0],begpos[1],begpos[2],curpos[0],curpos[1],curpos[2]) > 0.05){
      p3d_get_robot_config_into(MHP_GIK->robot,&robotq); 

      /* robotq contains robot's configuration to be written into the poster */
      
#ifdef JIDO
//      for(i=0; i<6; i++)    
//	SDI_F->Path.trajectory[i][Pathpointindex] = robotq[12+i];        //Jido specific
//      
//      Pathpointindex++;
//      SDI_F->Path.nb_positions = Pathpointindex;
#elif defined(HRP2)
      
      /* TODO: --- HRP2 functions ---- */
#elif defined(BH2)
#elif defined(PR2)
#elif defined(SIMULATION)
#else
#error "no poster writing function has been declared for the current robot"
#endif
      p3d_mat4ExtractPosReverseOrder(MHP_GIK->robot->joints[GIK_TASKS[target->motiontype].active_joint]->abs_pos, begpos, begpos+1, begpos+2,begpos+3, begpos+4, begpos+5);
    }
    
    maxdistance = 0;
    for(i=0; i<MHP_GIK->task_no; i++){  
      remainingdist = hri_gik_remainingdistance(MHP_GIK,i);
      if(remainingdist > maxdistance)
	maxdistance = remainingdist;
      DEBUG_PRINTF(4,("Remaining distance for task %d: %f\n",i, remainingdist));   
    }
    
    max_count++;    
    
  }while( (max_count < 100) &&
	  (maxdistance > 0.05) );
  
  gsl_vector_free(DT); /* FREE */
  MY_FREE(robotq, double, MHP_GIK->robot->nb_dof); /* FREE */
  
  if(maxdistance > 0.05){
    printf("**MHP** Warning planning couldn't reach the goal. Poster not complete\n ");
    *report = S_mhp_NOT_REACHED;
  }
  
#ifdef JIDO
  //  mhpPathPathPosterWrite ( MHP_PATH_POSTER_ID, &SDI_F->Path );
#endif  
  
  SDI_F->State.planning = FALSE;
  
  /* ------ End of Planning ------ */
  
  return ETHER;
}


/* mhpPlanPathInter  -  codel INTER of PlanPath
   Returns:  INTER ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
mhpPlanPathInter(MHP_MOTION_TARGET *target, int *report)
{
  SDI_F->State.planning = FALSE;
  
  return ETHER;
}


#if defined (JIDO) || defined (PR2)
////////////////////////////////////////////////////////////
///                   XAV's REQUESTS                     ///
////////////////////////////////////////////////////////////


/* mhpCleanRoadmapMain  -  codel EXEC of CleanRoadmap
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
mhpCleanRoadmapMain(int *report)
{
  SDI_F->State.planning = TRUE;

 SDI_F->collisionState.state = GENMANIP_COLLISION_FREE;


  if (MANIP_PLANNER== NULL) {
    *report = S_mhp_NOT_INITIALIZED;
    return ETHER;
  }
  MANIP_PLANNER->cleanRoadmap();
  printf("clean roadmap ok\n");
  MANIP_PLANNER->cleanTraj();
  printf("clean traj ok\n");
  SDI_F->State.planning = FALSE;
  return ETHER;
}



/*------------------------------------------------------------------------
 * ArmPlanTask
 *
 * Description: 
 *
 * Reports:      OK
 *              S_mhp_NOT_INITIALIZED
 *              S_mhp_NO_TRAJ_FOUND
 *              S_mhp_INVALID_QSTART
 *              S_mhp_INVALID_QGOAL
 *              S_mhp_INVALID_TRAJ_ID
 *              S_mhp_INVALID_TASK
 *              S_mhp_UNKNOWN_OBJECT
 *              S_mhp_NO_GRASP
 *              S_mhp_NO_PLACE
 *              S_mhp_ERROR_UNKNOWN
 *              S_mhp_EQUAL_QSTART_QGOAL
 */

/* mhpArmPlanTaskStart  -  codel START of ArmPlanTask
   Returns:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
mhpArmPlanTaskStart(STRUCT_MHP_ARM_PLAN_PARAMS *armPlanTaskParams, int *report)
{
  configPt qstart= NULL, qgoal= NULL;

  if(!mhp_fetch_and_update_env(report))
    printf("Warning: Cannot fetch the current environment\n");
 
  SDI_F->State.planning = TRUE; /* --- Planning Mode ---- */

  SDI_F->collisionState.state = GENMANIP_COLLISION_FREE;

  if (MANIP_PLANNER== NULL) {
    *report = S_mhp_NOT_INITIALIZED;
    return ETHER;
 }

  switch (armPlanTaskParams->taskType) {
  case MHP_ARM_FREE :
  case MHP_ARM_PICK_GOTO :
  case MHP_ARM_TAKE_TO_FREE :
  case MHP_ARM_TAKE_TO_PLACE :
  case MHP_ARM_PLACE_FROM_FREE :
  case MHP_ARM_EXTRACT :
  case MHP_ARM_ESCAPE_OBJECT :
    break;
  default :
    *report = S_mhp_INVALID_TASK;
    return END;
  }
  
  if(armPlanTaskParams->cartesian == GEN_TRUE){
    printf("cartesian mode\n");
    MANIP_PLANNER->setArmCartesian(0, true);
  } else {
    MANIP_PLANNER->setArmCartesian(0, false);
  }

  // mhpInitRobotPoseForPlanning
  XYZ_ENV->cur_robot= MHP_ROBOT;
  //printf("your robot is %s\n",MHP_ROBOT->name); 
  qstart= p3d_alloc_config(MHP_ROBOT);
  qgoal= p3d_alloc_config(MHP_ROBOT);
  
  p3d_get_robot_config_into(MHP_ROBOT, &qstart);


  p3d_update_virtual_object_config_for_arm_ik_constraint(MHP_ROBOT,0, qstart);

  
  if(armPlanTaskParams->startIsCurrent == GEN_FALSE){

    mhp_Arm2ConfigPt(&armPlanTaskParams->armStart, qstart);
    p3d_update_virtual_object_config_for_arm_ik_constraint(MHP_ROBOT,0, qstart);
    p3d_get_robot_config_into(MHP_ROBOT, &qstart);
  }
  p3d_copy_config_into(MHP_ROBOT, qstart, &qgoal);
  //printf("conf start\n"); 
  //print_config(MHP_ROBOT, qstart);

    mhp_Arm2ConfigPt(&armPlanTaskParams->armGoto, qgoal);

    /************************/
    /* Fix the circular jnt */
    /************************/
    p3d_jnt * jntPt = NULL;
    double vmin=0.0,vmax=0.0;
    double diff=0.0;
    int k=0;

    int njnt = MHP_ROBOT->njoints;
    /* translation parameters of main body */
    for (int i=0; i<=njnt; i++) {
      jntPt = MHP_ROBOT->joints[i];
      if(jntPt->type == P3D_ROTATE) {
	for (int j=0; j<jntPt->dof_equiv_nbr; j++) {
	  k = jntPt->index_dof+j;
	  if (p3d_jnt_is_dof_circular(jntPt, j)){
	    double tmp = qgoal[k];
	    qgoal[k] =  angle_limit_PI(qgoal[k]);
	    //printf("limit dof %d from %f to %f \n",k,tmp , q[k]);
	  }
	}
      }
    }
    
    if(p3d_isOutOfBounds(MHP_ROBOT, qgoal)){
	printf("The qgoal of %s is out of bounds\n", MHP_ROBOT->name);
    }

  p3d_update_virtual_object_config_for_arm_ik_constraint(MHP_ROBOT,0, qgoal);

  p3d_copy_config_into(MHP_ROBOT, qstart, &MHP_ROBOT->ROBOT_POS);
  p3d_copy_config_into(MHP_ROBOT, qgoal, &MHP_ROBOT->ROBOT_GOTO);

  p3d_set_and_update_this_robot_conf(MHP_ROBOT, qstart);
  p3d_destroy_config(MHP_ROBOT, qstart);
  p3d_destroy_config(MHP_ROBOT, qgoal);

  //printf("conf MHP_ROBOT->ROBOT_POS\n"); 
  //print_config(MHP_ROBOT, MHP_ROBOT->ROBOT_POS);
  //printf("conf MHP_ROBOT->ROBOT_GOTO\n"); 
  //print_config(MHP_ROBOT, MHP_ROBOT->ROBOT_GOTO);
  return EXEC;
}

/* mhpArmPlanTaskMain  -  codel EXEC of ArmPlanTask
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
mhpArmPlanTaskMain(STRUCT_MHP_ARM_PLAN_PARAMS *armPlanTaskParams, int *report)
{
  int result;
  int i =0;
  int indexTraj = armPlanTaskParams->trajId;
  std::vector <MANPIPULATION_TRAJECTORY_CONF_STR> confs;
  std::vector <SM_TRAJ> smTrajs;
  std::vector<double> objStart,objGoto;

  if(indexTraj < 0 || indexTraj > MHP_ARM_NB_MAX_TRAJ) {
    *report = S_mhp_INVALID_TRAJ_ID;
    return END;
  }
  if(armPlanTaskParams->cartesian == GEN_TRUE) {
    printf("cartesian mode\n");
    MANIP_PLANNER->setArmCartesian(0, true);
  } else {
    MANIP_PLANNER->setArmCartesian(0, false);
  }
  objStart.resize(6);
  objGoto.resize(6);
  for(int i = 0; i < 6; i++){
    objStart[i] = objGoto[i] = P3D_HUGE;
  }
  //Traslations
 if(armPlanTaskParams->useObjStart & 1){
    objStart[0] = armPlanTaskParams->objStart[0];
    objStart[1] = armPlanTaskParams->objStart[1];
    objStart[2] = armPlanTaskParams->objStart[2];
//    Bit computation to get the rotations to sample.
//    When the Bit = 1 the corresponding rotation has to be sampled
    if(~armPlanTaskParams->useObjStart & 2){
      objStart[3] = armPlanTaskParams->objStart[3];
    }
    if(~armPlanTaskParams->useObjStart & 4){
      objStart[4] = armPlanTaskParams->objStart[4];
    }
    if(~armPlanTaskParams->useObjStart & 8){
      objStart[5] = armPlanTaskParams->objStart[5];
    }
  }
  if(armPlanTaskParams->useObjGoto & 1){
    objGoto[0] = armPlanTaskParams->objGoto[0];
    objGoto[1] = armPlanTaskParams->objGoto[1];
    objGoto[2] = armPlanTaskParams->objGoto[2];
//    Bit computation to get the rotations to sample.
//    When the Bit = 1 the corresponding rotation has to be sampled
    if(~armPlanTaskParams->useObjGoto & 2){
      objGoto[3] = armPlanTaskParams->objGoto[3];
    }
    if(~armPlanTaskParams->useObjGoto & 4){
      objGoto[4] = armPlanTaskParams->objGoto[4];
    }
    if(~armPlanTaskParams->useObjGoto & 8){
      objGoto[5] = armPlanTaskParams->objGoto[5];
    }
  }

#ifdef HRI_COSTSPACE
  if(SDI_F->use_hri == GEN_TRUE )
    {
      ENV.setBool(Env::isCostSpace,true);
      MANIP_PLANNER->setPlanningMethod( planner_Function );
      MANIP_PLANNER->setSmoothingMethod( smoothing_Function );
    }
  else
    {  
      ENV.setBool(Env::isCostSpace,false);
      MANIP_PLANNER->resetPlanningMethod();
      MANIP_PLANNER->resetSmoothingMethod();
    }
#endif
#ifdef PR2
    fixAllJointsWithoutArm(MHP_ROBOT, 0);
#endif

  result = MANIP_PLANNER->armPlanTask(MANIPULATION_TASK_TYPE_STR(armPlanTaskParams->taskType), 0, 
				   MANIP_PLANNER->robotStart(),MANIP_PLANNER->robotGoto(),objStart, objGoto, (char*)armPlanTaskParams->objectName.name, (char*)armPlanTaskParams->supportName.name, (char*)armPlanTaskParams->placementName.name, confs, smTrajs);
  switch(result){
    case MANIPULATION_TASK_OK:{
      break;
    }
    case MANIPULATION_TASK_NOT_INITIALIZED:{
      *report= S_mhp_NOT_INITIALIZED;
      return END;
    }
    case MANIPULATION_TASK_NO_TRAJ_FOUND:{
      *report= S_mhp_NO_TRAJ_FOUND;
      return END;
    }
    case MANIPULATION_TASK_INVALID_QSTART:{
      *report= S_mhp_INVALID_QSTART;
      return END;
    }
    case MANIPULATION_TASK_INVALID_QGOAL:{
      *report= S_mhp_INVALID_QGOAL;
      return END;
    }
    case MANIPULATION_TASK_INVALID_TRAJ_ID:{
      *report= S_mhp_INVALID_TRAJ_ID;
      return END;
    }
    case MANIPULATION_TASK_INVALID_TASK:{
      *report= S_mhp_INVALID_TASK;
      return END;
    }
    case MANIPULATION_TASK_UNKNOWN_OBJECT:{
      *report= S_mhp_UNKNOWN_OBJECT;
      return END;
    }
    case MANIPULATION_TASK_NO_GRASP:{
      *report= S_mhp_NO_GRASP;
      return END;
    }
    case MANIPULATION_TASK_NO_PLACE:{
      *report= S_mhp_NO_PLACE;
      return END;
    }
    case MANIPULATION_TASK_ERROR_UNKNOWN:{
      *report= S_mhp_ERROR_UNKNOWN;
      return END;
    }
    case MANIPULATION_TASK_EQUAL_QSTART_QGOAL:{
      *report= S_mhp_EQUAL_QSTART_QGOAL;
      return END;
    }
  }

  
  smTrajs[0].convertToSM_TRAJ_STR(&(SDI_F->armTrajArray.traj[indexTraj]));
  ++SM_TRAJ_ID;
  SDI_F->armTrajArray.traj[indexTraj].trajId = SM_TRAJ_ID;
  printf("smTraj.nbAxis %d\n", SDI_F->armTrajArray.traj[indexTraj].nbAxis);

  mhpArmTrajarmTrajectoryOuputPosterWrite( MHP_ARMTRAJ_POSTER_ID, &(SDI_F->armTrajArray.traj[indexTraj]));
  return END;
}



/* mhpArmPlanTaskEnd  -  codel END of ArmPlanTask
   Returns:  END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
mhpArmPlanTaskEnd(STRUCT_MHP_ARM_PLAN_PARAMS *armPlanTaskParams, int *report)
{
  deactivateCcCntrts(MHP_ROBOT, -1);
  SDI_F->State.planning = FALSE;
  return ETHER;
}

/* mhpArmPlanTaskInter  -  codel INTER of ArmPlanTask
   Returns:  INTER ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
mhpArmPlanTaskInter(STRUCT_MHP_ARM_PLAN_PARAMS *armPlanTaskParams, int *report)
{
  SDI_F->State.planning = FALSE;
  return ETHER;
}



/*------------------------------------------------------------------------
 * ArmSelectTraj
 *
 * Description: 
 *
 * Reports:      OK
 *              S_mhp_NOT_INITIALIZED
 *              S_mhp_INVALID_TRAJ_ID
 *              S_mhp_NO_TRAJ
 */

/* mhpArmSelectTrajStart  -  codel START of ArmSelectTraj
   Returns:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
mhpArmSelectTrajStart(int *currentTrajId, int *report)
{
  SDI_F->State.planning = TRUE; /* --- Planning Mode ---- */
  int indexTraj = *currentTrajId;
  if (MANIP_PLANNER== NULL) {
    *report = S_mhp_NOT_INITIALIZED;
    return END;
  }
  if(indexTraj < 0 || indexTraj > MHP_ARM_NB_MAX_TRAJ) {
    *report = S_mhp_INVALID_TRAJ_ID;
    return END;
  }

  return EXEC;
}

/* mhpArmSelectTrajMain  -  codel EXEC of ArmSelectTraj
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
mhpArmSelectTrajMain(int *currentTrajId, int *report)
{
  int indexTraj = *currentTrajId;

  mhpArmTrajarmTrajectoryOuputPosterWrite( MHP_ARMTRAJ_POSTER_ID, &SDI_F->armTrajArray.traj[indexTraj]);
  return END;
}

/* mhpArmSelectTrajEnd  -  codel END of ArmSelectTraj
   Returns:  END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
mhpArmSelectTrajEnd(int *currentTrajId, int *report)
{
  SDI_F->State.planning = FALSE;
  return ETHER;
}

/* mhpArmSelectTrajInter  -  codel INTER of ArmSelectTraj
   Returns:  INTER ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
mhpArmSelectTrajInter(int *currentTrajId, int *report)
{
  SDI_F->State.planning = FALSE;
  return ETHER;
}




/*------------------------------------------------------------------------
 * ArmGetTrajInfo
 *
 * Description: 
 *
 * Reports:      OK
 *              S_mhp_NOT_INITIALIZED
 *              S_mhp_INVALID_TRAJ_ID
 *              S_mhp_NO_TRAJ
 */

/* mhpArmGetTrajInfoStart  -  codel START of ArmGetTrajInfo
   Returns:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
mhpArmGetTrajInfoStart(int *currentTrajId, STRUCT_TRAJ_INFO *armTrajInfo, int *report)
{
   SDI_F->State.planning = TRUE; /* --- Planning Mode ---- */
  int indexTraj = *currentTrajId;
  if (MANIP_PLANNER== NULL) {
    *report = S_mhp_NOT_INITIALIZED;
    return END;
  }
  if(indexTraj < 0 || indexTraj > MHP_ARM_NB_MAX_TRAJ) {
    *report = S_mhp_INVALID_TRAJ_ID;
    return END;
  }

  return EXEC;
}

/* mhpArmGetTrajInfoMain  -  codel EXEC of ArmGetTrajInfo
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
mhpArmGetTrajInfoMain(int *currentTrajId, STRUCT_TRAJ_INFO *armTrajInfo, int *report)
{
  double RX=0.0, RY=0.0, RZ=0.0;
   configPt qstart = NULL;
  // mhpInitRobotPoseForPlanning
  XYZ_ENV->cur_robot= MHP_ROBOT;
  printf("your robot is %s\n",MHP_ROBOT->name); 
  qstart= p3d_alloc_config(MHP_ROBOT);
 
  p3d_get_robot_config_into(MHP_ROBOT, &qstart);

  p3d_update_virtual_object_config_for_arm_ik_constraint(MHP_ROBOT,0, qstart);


  
  int result = 0;
  int trajId = *currentTrajId;

  //int indexTraj = SDI_F->armTrajArray.traj[trajId].nb_points -1;
  //
  //armTrajInfo->armQGoto.q1 =  SDI_F->armTrajArray.traj[trajId].positions[indexTraj].q1;
  //armTrajInfo->armQGoto.q2 =  SDI_F->armTrajArray.traj[trajId].positions[indexTraj].q2;
  //armTrajInfo->armQGoto.q3 =  SDI_F->armTrajArray.traj[trajId].positions[indexTraj].q3;
  //armTrajInfo->armQGoto.q4 =  SDI_F->armTrajArray.traj[trajId].positions[indexTraj].q4;
  //armTrajInfo->armQGoto.q5 =  SDI_F->armTrajArray.traj[trajId].positions[indexTraj].q5;
  //armTrajInfo->armQGoto.q6 =  SDI_F->armTrajArray.traj[trajId].positions[indexTraj].q5;
  //armTrajInfo->armQGoto.q7 =  SDI_F->armTrajArray.traj[trajId].positions[indexTraj].q7;

  //MANIP_PLANNER->setArmQ( SDI_F->armTrajArray.traj[trajId].positions[indexTraj].q1,  SDI_F->armTrajArray.traj[trajId].positions[indexTraj].q2,
  //      SDI_F->armTrajArray.traj[trajId].positions[indexTraj].q3,  SDI_F->armTrajArray.traj[trajId].positions[indexTraj].q4,
  //     SDI_F->armTrajArray.traj[trajId].positions[indexTraj].q5,  SDI_F->armTrajArray.traj[trajId].positions[indexTraj].q6,  SDI_F->armTrajArray.traj[trajId].positions[indexTraj].q7);


  //mhp_Arm2ConfigPt(SDI_F->armTrajArray.traj[trajId].positions[indexTraj], qstart);
  //p3d_update_virtual_object_config_for_arm_ik_constraint(MHP_ROBOT,0, qstart);
  //p3d_get_robot_config_into(MHP_ROBOT, &qstart);
  //
  //MANIP_PLANNER->getArmX(&armTrajInfo->armXAbsGoto.x, &armTrajInfo->armXAbsGoto.y,&armTrajInfo->armXAbsGoto.z, &RX, &RY, &RZ);
  p3d_destroy_config(MHP_ROBOT, qstart);

  return END;
}

/* mhpArmGetTrajInfoEnd  -  codel END of ArmGetTrajInfo
   Returns:  END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
mhpArmGetTrajInfoEnd(int *currentTrajId, STRUCT_TRAJ_INFO *armTrajInfo, int *report)
{
  SDI_F->State.planning = FALSE;
  return ETHER;
}

/* mhpArmGetTrajInfoInter  -  codel INTER of ArmGetTrajInfo
   Returns:  INTER ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
mhpArmGetTrajInfoInter(int *currentTrajId, STRUCT_TRAJ_INFO *armTrajInfo, int *report)
{
  SDI_F->State.planning = FALSE;
  return ETHER;
}



/*------------------------------------------------------------------------
 * ArmSetSupport
 *
 * Description: 
 *
 * Reports:      OK
 *              S_mhp_NOT_INITIALIZED
 *              S_mhp_INVALID_SUPPORT
 */

/* mhpArmSetSupportMain  -  codel EXEC of ArmSetSupport
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
mhpArmSetSupportMain(GEN_STRING128 *support, int *report)
{

  SDI_F->State.planning = TRUE; /* --- Planning Mode ---- */
  if (MANIP_PLANNER== NULL) {
    *report = S_mhp_NOT_INITIALIZED;
    return END;
  }

//  if(MANIP_PLANNER->setSupport(support->name)!=0) {
//    printf("ERROR cannot set the support to %s\n",support->name);
//    *report = S_mhp_INVALID_SUPPORT;
//  }

  SDI_F->State.planning = FALSE;

  return ETHER;
}


/*------------------------------------------------------------------------
 * ArmCheckCollisionOnTraj
 *
 * Description: 
 *
 * Reports:      OK
 *              S_mhp_NOT_INITIALIZED
 */

/* mhpArmCheckCollisionOnTrajStart  -  codel START of ArmCheckCollisionOnTraj
   Returns:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
mhpArmCheckCollisionOnTrajStart(GEN_STRING128 *posterArmPoseOnTrajName, int *report)
{
  SDI_F->State.planning = TRUE; /* --- Planning Mode ---- */
  GENMANIP_POSITION_STR poseOnTraj;
  
  if (MANIP_PLANNER== NULL) {
    *report = S_mhp_NOT_INITIALIZED;
    return END;
  }
  if(mhpGENMANIP_POSITION_STRPosterFind(posterArmPoseOnTrajName->name, &posterArmPoseOnTraj)!=OK) {
    *report = S_mhp_POSTER_NOT_FOUND;
    return END;
  }

  if(mhpGENMANIP_POSITION_STRPosterRead(posterArmPoseOnTraj, &poseOnTraj)!=OK) {
    *report = S_mhp_POSTER_NOT_READ;
    return END;
  }
  SDI_F->collisionState.state = GENMANIP_COLLISION_FREE;
  SDI_F->collisionState.mode = GENMANIP_ON;
  return EXEC;
}

/* mhpArmCheckCollisionOnTrajMain  -  codel EXEC of ArmCheckCollisionOnTraj
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
mhpArmCheckCollisionOnTrajMain(GEN_STRING128 *posterArmPoseOnTrajName, int *report)
{
  SDI_F->State.planning = TRUE; /* --- Planning Mode ---- */
  GENMANIP_POSITION_STR poseOnTraj;
  
  if(!mhp_fetch_and_update_env(report))
    printf("Warning: Cannot fetch the current environment\n");
  
  if(mhpGENMANIP_POSITION_STRPosterRead(posterArmPoseOnTraj, &poseOnTraj)!=OK) {
    *report = S_mhp_POSTER_NOT_READ;
    return END;
  }
#ifdef DPG  
  if(MANIP_PLANNER->checkCollisionOnTraj(poseOnTraj.lp)==0) {
    SDI_F->collisionState.state = GENMANIP_COLLISION_FREE;
  } else {
    SDI_F->collisionState.state = GENMANIP_COLLISION_COLLISION;
    return END;
    /*   MANIP_PLANNER->replanCollidingTraj(MHP_ROBOT, 0, poseOnTraj->lp, );*/
  }
#else
  printf("flag DPG is not defined\n");
#endif

  SDI_F->State.planning = FALSE;
  return EXEC;
}

/* mhpArmCheckCollisionOnTrajEnd  -  codel END of ArmCheckCollisionOnTraj
   Returns:  END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
mhpArmCheckCollisionOnTrajEnd(GEN_STRING128 *posterArmPoseOnTrajName, int *report)
{
  
  SDI_F->State.planning = FALSE;
  return ETHER;
}

/*------------------------------------------------------------------------
 * ArmRePlanColTraj
 *
 * Description: 
 *
 * Reports:      OK
 *              S_mhp_NOT_INITIALIZED
 */

/* mhpArmRePlanColTrajStart  -  codel START of ArmRePlanColTraj
   Returns:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
mhpArmRePlanColTrajStart(GEN_STRING128 *posterArmPoseOnTrajName, int *report)
{
  if(!mhp_fetch_and_update_env(report))
    printf("Warning: Cannot fetch the current environment\n");

  SDI_F->State.planning = TRUE; /* --- Planning Mode ---- */
  GENMANIP_POSITION_STR poseOnTraj;

  if (MANIP_PLANNER== NULL) {
    *report = S_mhp_NOT_INITIALIZED;
    return END;
  }

  if(mhpGENMANIP_POSITION_STRPosterFind(posterArmPoseOnTrajName->name, &posterArmPoseOnTraj)!=OK)
 {
    *report = S_mhp_POSTER_NOT_FOUND;
    return END;
  }
/*  if(SDI_F->collisionState.state == GENMANIP_COLLISION_FREE) {
    return END;
  }*/
  // printf("Replaning Start OK\n");
  return EXEC;
}

/* mhpArmRePlanColTrajMain  -  codel EXEC of ArmRePlanColTraj
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
mhpArmRePlanColTrajMain(GEN_STRING128 *posterArmPoseOnTrajName, int *report)
{
  GENMANIP_POSITION_STR poseOnTraj;
  int indexTraj = 1;
  int result;
  std::vector <MANPIPULATION_TRAJECTORY_CONF_STR> confs;
  std::vector <SM_TRAJ> smTrajs;

  if(mhpGENMANIP_POSITION_STRPosterRead(posterArmPoseOnTraj, &poseOnTraj)!=OK) {
    *report = S_mhp_POSTER_NOT_READ;
    return END;
  }
  printf("Replanning\n");

#ifdef DPG
  result = MANIP_PLANNER->replanCollidingTraj(poseOnTraj.lp, confs, smTrajs);

  if(result!=0) {
    *report= S_mhp_NO_PATH_FOUND;
    return END;
  }

#else
  printf("flag DPG is not defined\n");
#endif


//  for(unsigned int i = 0; i < MANIP_PLANNER->positions.size(); i++){
//     SDI_F->armTrajArray.traj[indexTraj].positions[i].q1 =  MANIP_PLANNER->positions[i][0];
//     SDI_F->armTrajArray.traj[indexTraj].positions[i].q2 =  MANIP_PLANNER->positions[i][1];
//     SDI_F->armTrajArray.traj[indexTraj].positions[i].q3 =  MANIP_PLANNER->positions[i][2];
//     SDI_F->armTrajArray.traj[indexTraj].positions[i].q4 =  MANIP_PLANNER->positions[i][3];
//     SDI_F->armTrajArray.traj[indexTraj].positions[i].q5 =  MANIP_PLANNER->positions[i][4];
//     SDI_F->armTrajArray.traj[indexTraj].positions[i].q6 =  MANIP_PLANNER->positions[i][5];
//     SDI_F->armTrajArray.traj[indexTraj].positions[i].q7 =  MANIP_PLANNER->positions[i][6];
//     SDI_F->armTrajArray.traj[indexTraj].lp[i] =  MANIP_PLANNER->lp[i];
//  }
//  SDI_F->armTrajArray.traj[indexTraj].nb_points =  MANIP_PLANNER->positions.size();
//  //printf("TRAJECTORY :\n");
//  //printf("nbPoints = %d\n",SDI_F->armTrajArray.traj[indexTraj].nb_points);
//  // for(int i = 0; i<SDI_F->armTrajArray.traj[indexTraj].nb_points; i++) {
//  //  printf("lpId %d q[%d] %f %f %f %f %f %f %f\n",  SDI_F->armTrajArray.traj[indexTraj].lp[i],i, 
//  //	   SDI_F->armTrajArray.traj[indexTraj].positions[i].q1, SDI_F->armTrajArray.traj[indexTraj].positions[i].q2,
//  //   SDI_F->armTrajArray.traj[indexTraj].positions[i].q3, SDI_F->armTrajArray.traj[indexTraj].positions[i].q4, 
//  //   SDI_F->armTrajArray.traj[indexTraj].positions[i].q5, SDI_F->armTrajArray.traj[indexTraj].positions[i].q6, SDI_F->armTrajArray.traj[indexTraj].positions[i].q7);
//    
//  // }
//  mhpArmTrajarmTrajectoryOuputPosterWrite( MHP_ARMTRAJ_POSTER_ID, &SDI_F->armTrajArray.traj[indexTraj]);
  return END;
}

/* mhpArmRePlanColTrajEnd  -  codel END of ArmRePlanColTraj
   Returns:  END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
mhpArmRePlanColTrajEnd(GEN_STRING128 *posterArmPoseOnTrajName, int *report)
{
  SDI_F->collisionState.state = GENMANIP_COLLISION_FREE;
  SDI_F->State.planning = FALSE;
  return ETHER;
}



/*------------------------------------------------------------------------
 * ArmComputePRM
 *
 * Description: 
 *
 * Reports:      OK
 *              S_mhp_NOT_INITIALIZED
 *              S_mhp_WRONG_PARAMETER
 */

/* mhpArmComputePRMStart  -  codel START of ArmComputePRM
   Returns:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
mhpArmComputePRMStart(double *prmComputationTime, int *report)
{
  configPt qstart= NULL;

  if(!mhp_fetch_and_update_env(report))
    printf("Warning: Cannot fetch the current environment\n");

  SDI_F->State.planning = TRUE; /* --- Planning Mode ---- */

  SDI_F->collisionState.state = GENMANIP_COLLISION_FREE;

  if( *prmComputationTime < 0.0) {
    // *report = S_mhp_WRONG_PARAMETER;
    return END;
  }

  if (MANIP_PLANNER== NULL) {
    *report = S_mhp_NOT_INITIALIZED;
    return END;
  }

  XYZ_ENV->cur_robot= MHP_ROBOT;
  //  printf("your robot is %s\n",MHP_ROBOT->name); 
  qstart= p3d_alloc_config(MHP_ROBOT);
  p3d_get_robot_config_into(MHP_ROBOT, &qstart);
  p3d_update_virtual_object_config_for_arm_ik_constraint(MHP_ROBOT,0, qstart);
//  qstart[21] =  0.0325;
//  qstart[22] =  0.0325;
  p3d_copy_config_into(MHP_ROBOT, qstart, &MHP_ROBOT->ROBOT_POS);
  p3d_copy_config_into(MHP_ROBOT, qstart, &MHP_ROBOT->ROBOT_GOTO);
  p3d_set_and_update_this_robot_conf(MHP_ROBOT, qstart);
  p3d_destroy_config(MHP_ROBOT, qstart);

  //  printf("ArmComputePRMStart OK\n");
  return EXEC;
}

/* mhpArmComputePRMMain  -  codel EXEC of ArmComputePRM
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
mhpArmComputePRMMain(double *prmComputationTime, int *report)
{
  //MANIP_PLANNER->setArmCartesian(false);

  
  MANIP_PLANNER->armComputePRM(*prmComputationTime);

  return END;
}

/* mhpArmComputePRMEnd  -  codel END of ArmComputePRM
   Returns:  END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
mhpArmComputePRMEnd(double *prmComputationTime, int *report)
{

  SDI_F->State.planning = FALSE;
  return ETHER;
}




#endif


#if defined(USE_MIGHTABILITY_MAPS)
/* ------- START: Planning Requests for HRI Task ------- */


ACTIVITY_EVENT
mhpPlan_HRI_TaskStart(STRUCT_HRI_TASK_REQ *HRI_task_IP,int *report)
{
  printf(" Trying mhp_fetch_and_update_env(report)\n");
  if(!mhp_fetch_and_update_env(report))
  {
    printf(">>>>> HRI TASK INIT ERROR: Cannot fetch the current environment from SPARK, so there will risk of inconsistency during real execution so planner will NOT Plan.\n");
    return END;
  }
  
  
  return EXEC;
}

ACTIVITY_EVENT
mhpPlan_HRI_TaskMain(STRUCT_HRI_TASK_REQ *HRI_task_IP,int *report)
{
  printf(" >*>*>*> Inside mhpPlan_HRI_TaskMain() with task name= %s, for object %s, by agent = %s, for agent= %s\n",HRI_task_IP->task.task_name,HRI_task_IP->task.for_object, HRI_task_IP->task.by_agent,HRI_task_IP->task.for_agent);
  
  set_all_Mightability_Analyses_to_update();

  execute_Mightability_Map_functions();

  HRI_task_desc task_to_validate;
  symbolic_HRI_task_desc symb_HRI_task_ip;

  strcpy(symb_HRI_task_ip.task_name,HRI_task_IP->task.task_name);
  strcpy(symb_HRI_task_ip.for_object,HRI_task_IP->task.for_object);
  strcpy(symb_HRI_task_ip.by_agent,HRI_task_IP->task.by_agent);
  strcpy(symb_HRI_task_ip.for_agent,HRI_task_IP->task.for_agent);

  printf(" >>>> Inside mhpPlan_HRI_TaskMain(), calling convert_symbolic_HRI_task_desc_into() with task name= %s, for object %s, by agent = %s, for agent= %s\n",symb_HRI_task_ip.task_name,symb_HRI_task_ip.for_object, symb_HRI_task_ip.by_agent,symb_HRI_task_ip.for_agent);

  if(convert_symbolic_HRI_task_desc_into(symb_HRI_task_ip, task_to_validate)<=0)
  {
   printf(" HRI_TASK ERROR : Could not convert the symbolic description of task to validate \n");
   return END;
  }
  IS_PERFORMING_AGENT_MASTER=HRI_task_IP->task.by_master;
  CONSIDER_OBJECT_DIMENSION_FOR_CANDIDATE_PTS=HRI_task_IP->task.use_obj_dimension;

  int val_task_res=validate_HRI_task(task_to_validate, HRI_task_IP->task_plan_id, HRI_task_IP->task.for_proactive_behavior);
  
  printf(" After validate_HRI_task \n");

  if(val_task_res==0)
  {
   *report = S_mhp_PLAN_HRI_TASK_FAIL;
  }
  return END;
}

ACTIVITY_EVENT
mhpPlan_HRI_TaskInter(STRUCT_HRI_TASK_REQ *HRI_task_IP,int *report)
{
    return END;
}

ACTIVITY_EVENT
mhpPlan_HRI_TaskEnd(STRUCT_HRI_TASK_REQ *HRI_task_IP,int *report)
{
    printf(" Inside mhpPlan_HRI_TaskEnd\n");
    return ETHER;
}

//To execute the p3d trajectories for a particular plan ID in simulation
ACTIVITY_EVENT
mhpExec_HRI_Task_P3D_Traj_SimuStart(STRUCT_PLAN_ID_IP_FOR_HRI_TASK *plan_ID_exec_p3d_traj,int *report)
{
  
  return EXEC;
}

ACTIVITY_EVENT
mhpExec_HRI_Task_P3D_Traj_SimuMain(STRUCT_PLAN_ID_IP_FOR_HRI_TASK *plan_ID_exec_p3d_traj,int *report)
{
  
 show_p3d_trajectories_for_plan_ID(plan_ID_exec_p3d_traj->task_plan_id);
  return END;
}

ACTIVITY_EVENT
mhpExec_HRI_Task_P3D_Traj_SimuInter(STRUCT_PLAN_ID_IP_FOR_HRI_TASK *plan_ID_exec_p3d_traj,int *report)
{
    return END;
}

ACTIVITY_EVENT
mhpExec_HRI_Task_P3D_Traj_SimuEnd(STRUCT_PLAN_ID_IP_FOR_HRI_TASK *plan_ID_exec_p3d_traj,int *report)
{
    printf(" Inside mhpExec_HRI_Task_P3D_Traj_SimuEnd\n");
    return ETHER;
}


//To get the Soft Motion trajectory for a particular plan ID
ACTIVITY_EVENT
mhpGet_SM_Traj_HRI_TaskStart(STRUCT_IP_FOR_HRI_TASK_SM *plan_ID_for_SM,int *report)
{
  
  return EXEC;
}

ACTIVITY_EVENT
mhpGet_SM_Traj_HRI_TaskMain(STRUCT_IP_FOR_HRI_TASK_SM *plan_ID_for_SM,int *report)
{
  curr_smTrajs.clear();
  get_soft_motion_trajectories_for_plan_ID(plan_ID_for_SM->task_plan_id, curr_smTrajs);

  printf(" Number of soft motion trajectories = %d \n",curr_smTrajs.size());
 
  return END;
}

ACTIVITY_EVENT
mhpGet_SM_Traj_HRI_TaskInter(STRUCT_IP_FOR_HRI_TASK_SM *plan_ID_for_SM,int *report)
{
    return END;
}

ACTIVITY_EVENT
mhpGet_SM_Traj_HRI_TaskEnd(STRUCT_IP_FOR_HRI_TASK_SM *plan_ID_for_SM,int *report)
{
    printf(" Inside mhpGet_SM_Traj_HRI_TaskEnd\n");
    return ETHER;
}

//To execute the current list of Soft Motion trajectories in simulation for visualization
ACTIVITY_EVENT
mhpExec_Curr_SM_Traj_SimuStart(int *report)
{
  if(curr_smTrajs.size()==0)
  {
  printf(" There is no Soft Motion Trajectory stored in the list \n");
  return END;
  } 
  return EXEC;
}

ACTIVITY_EVENT
mhpExec_Curr_SM_Traj_SimuMain(int *report)
{
  for(int i=0;i<curr_smTrajs.size();i++)
  {
   ececute_this_HRI_task_SM_Traj_in_simu("JIDOKUKA_ROBOT", curr_smTrajs[i]);
   ////return EXEC;
  }
 
  
  return END;
}

ACTIVITY_EVENT
mhpExec_Curr_SM_Traj_SimuInter(int *report)
{
    return END;
}

ACTIVITY_EVENT
mhpExec_Curr_SM_Traj_SimuEnd(int *report)
{
    printf(" Inside mhpExec_Curr_SM_Traj_SimuEnd\n");
    return ETHER;
}

//To write a particular Soft Motion trajectory to poster for execution
ACTIVITY_EVENT
mhpWrite_this_SM_Traj_to_PosterStart(STRUCT_SM_ID_IP_FOR_HRI_TASK *SM_traj_index_IP,int *report)
{
  if(SM_traj_index_IP->SM_traj_index>=curr_smTrajs.size())
  {
   printf(" HRI_TASK ERROR: Not a valid index for curr_smTrajs\n");
   return END;
  }
  return EXEC;
}

ACTIVITY_EVENT
mhpWrite_this_SM_Traj_to_PosterMain(STRUCT_SM_ID_IP_FOR_HRI_TASK *SM_traj_index_IP,int *report)
{
  curr_smTrajs[SM_traj_index_IP->SM_traj_index].convertToSM_TRAJ_STR(&curr_SM_traj_for_poster);
  printf("smTraj.nbAxis %d\n", curr_SM_traj_for_poster.nbAxis);

  mhpArmTrajarmTrajectoryOuputPosterWrite( MHP_ARMTRAJ_POSTER_ID, &curr_SM_traj_for_poster);
  return END;
}

ACTIVITY_EVENT
mhpWrite_this_SM_Traj_to_PosterInter(STRUCT_SM_ID_IP_FOR_HRI_TASK *SM_traj_index_IP,int *report)
{
    return END;
}

ACTIVITY_EVENT
mhpWrite_this_SM_Traj_to_PosterEnd(STRUCT_SM_ID_IP_FOR_HRI_TASK *SM_traj_index_IP,int *report)
{
    printf(" Inside mhpWrite_this_SM_Traj_to_PosterEnd\n");
    return ETHER;
}

//For HRI tasks requests from SHARY
ACTIVITY_EVENT
mhpPlan_HRI_Task_for_SHARYStart(STRUCT_HRI_TASK_IP_FROM_SHARY *TASK_IP_FROM_SHARY, STRUCT_HRI_TASK_TRAJ_OP_FOR_SHARY *TRAJ_OP_FOR_SHARY,int *report)
{
  printf(" Trying mhp_fetch_and_update_env(report)\n");
  if(!mhp_fetch_and_update_env(report))
  {
    printf(">>>>> HRI TASK INIT ERROR: Cannot fetch the current environment\n");
    *report=S_mhp_FAIL_TO_FETCH_ENV; 
    return END;
  }
  
  
  return EXEC;
}

ACTIVITY_EVENT
mhpPlan_HRI_Task_for_SHARYMain(STRUCT_HRI_TASK_IP_FROM_SHARY *TASK_IP_FROM_SHARY, STRUCT_HRI_TASK_TRAJ_OP_FOR_SHARY *TRAJ_OP_FOR_SHARY,int *report)
{
  printf(" >*>*>*> Inside mhpPlan_HRI_Task_for_SHARYMain() with task name= %s, for object %s, by agent = %s, for agent= %s\n",TASK_IP_FROM_SHARY->sh_task.task.task_name,TASK_IP_FROM_SHARY->sh_task.task.for_object, TASK_IP_FROM_SHARY->sh_task.task.by_agent,TASK_IP_FROM_SHARY->sh_task.task.for_agent);

  execute_Mightability_Map_functions();

  HRI_task_desc task_to_validate;
  symbolic_HRI_task_desc symb_HRI_task_ip;

  strcpy(symb_HRI_task_ip.task_name,TASK_IP_FROM_SHARY->sh_task.task.task_name);
  strcpy(symb_HRI_task_ip.for_object,TASK_IP_FROM_SHARY->sh_task.task.for_object);
  strcpy(symb_HRI_task_ip.by_agent,TASK_IP_FROM_SHARY->sh_task.task.by_agent);
  strcpy(symb_HRI_task_ip.for_agent,TASK_IP_FROM_SHARY->sh_task.task.for_agent);

  ////printf(" >>>> Inside mhpPlan_HRI_TaskMain(), calling convert_symbolic_HRI_task_desc_into() with task name= %s, for object %s, by agent = %s, for agent= %s\n",symb_HRI_task_ip.task_name,symb_HRI_task_ip.for_object, symb_HRI_task_ip.by_agent,symb_HRI_task_ip.for_agent);

  if(convert_symbolic_HRI_task_desc_into(symb_HRI_task_ip, task_to_validate)<=0)
  {
   printf(" HRI_TASK ERROR : Could not convert the symbolic description of task to validate \n");
   return END;
  }

  IS_PERFORMING_AGENT_MASTER=TASK_IP_FROM_SHARY->sh_task.task.by_master;
  int res=validate_HRI_task(task_to_validate, TASK_IP_FROM_SHARY->sh_task.task_plan_id, TASK_IP_FROM_SHARY->sh_task.task.for_proactive_behavior);
  
  printf(" After validate_HRI_task \n");
  if(res<=0)
  {
   printf(" HRI_TASK ERROR: Fail to validate current task\n");
   *report=S_mhp_Plan_HRI_Task_for_SHARY_FAIL; 
   return END;
  }

   std::vector <SM_TRAJ> smTrajs;
   
   int sub_traj_st_index=0;
   int sub_traj_end_index=2;
   int indexTraj=TASK_IP_FROM_SHARY->traj_info[0].traj_index;
  ////curr_smTrajs.clear();
  

  int SM_traj_res=get_single_soft_motion_traj_for_SHARY(TASK_IP_FROM_SHARY->sh_task.task_plan_id, sub_traj_st_index, sub_traj_end_index, smTrajs[0] );
  if(SM_traj_res<=0)
  {
   printf(" HRI_TASK ERROR : Fail to get Soft Motion Trajectory with concatTrajectories \n");
   *report=S_mhp_Plan_HRI_Task_for_SHARY_FAIL; 
   return END;
  }

  smTrajs[0].convertToSM_TRAJ_STR(&(SDI_F->armTrajArray.traj[indexTraj]));

   sub_traj_st_index=3;
    sub_traj_end_index=4;
    indexTraj=TASK_IP_FROM_SHARY->traj_info[1].traj_index;

    SM_traj_res=get_single_soft_motion_traj_for_SHARY(TASK_IP_FROM_SHARY->sh_task.task_plan_id, sub_traj_st_index, sub_traj_end_index, smTrajs[1] );
  if(SM_traj_res<=0)
  {
   printf(" HRI_TASK ERROR : Fail to get Soft Motion Trajectory with concatTrajectories \n");
   *report=S_mhp_Plan_HRI_Task_for_SHARY_FAIL; 
   return END;
  }

  smTrajs[1].convertToSM_TRAJ_STR(&(SDI_F->armTrajArray.traj[indexTraj]));

   sub_traj_st_index=5;
    sub_traj_end_index=6;
    indexTraj=TASK_IP_FROM_SHARY->traj_info[2].traj_index;
  if(indexTraj<0)
  {
  return END;
  }

    SM_traj_res=get_single_soft_motion_traj_for_SHARY(TASK_IP_FROM_SHARY->sh_task.task_plan_id, sub_traj_st_index, sub_traj_end_index, smTrajs[2] );
  if(SM_traj_res<=0)
  {
   printf(" HRI_TASK ERROR : Fail to get Soft Motion Trajectory with concatTrajectories \n");
   *report=S_mhp_Plan_HRI_Task_for_SHARY_FAIL; 
   return END;
  }

  smTrajs[2].convertToSM_TRAJ_STR(&(SDI_F->armTrajArray.traj[indexTraj]));

  ////printf(" Number of soft motion trajectories = %d \n",curr_smTrajs.size());

  return END;
}

ACTIVITY_EVENT
mhpPlan_HRI_Task_for_SHARYInter(STRUCT_HRI_TASK_IP_FROM_SHARY *TASK_IP_FROM_SHARY, STRUCT_HRI_TASK_TRAJ_OP_FOR_SHARY *TRAJ_OP_FOR_SHARY,int *report)
{
    return END;
}

ACTIVITY_EVENT
mhpPlan_HRI_Task_for_SHARYEnd(STRUCT_HRI_TASK_IP_FROM_SHARY *TASK_IP_FROM_SHARY, STRUCT_HRI_TASK_TRAJ_OP_FOR_SHARY *TRAJ_OP_FOR_SHARY,int *report)
{
    printf(" Inside mhpPlan_HRI_Task_for_SHARYEnd\n");
    return ETHER;
}


//For getting points for making accessible
candidate_poins_for_task *curr_resultant_candidate_points;

ACTIVITY_EVENT
mhpGetPointsToMakeAccessibleStart(STRUCT_POINTS_FOR_HRI_TASKS *points_to_make_accessible,int *report)
{
  printf(" Trying mhp_fetch_and_update_env(report)\n");
  if(!mhp_fetch_and_update_env(report))
  {
    printf(">>>>> HRI TASK INIT ERROR: Cannot fetch the current environment\n");
    *report=S_mhp_FAIL_TO_FETCH_ENV; 
    return END;
  }
  
   curr_resultant_candidate_points=MY_ALLOC(candidate_poins_for_task,1);
 
  return EXEC;
}

ACTIVITY_EVENT
mhpGetPointsToMakeAccessibleMain(STRUCT_POINTS_FOR_HRI_TASKS *points_to_make_accessible,int *report)
{
  printf(" >*>*>*> Inside mhpGetPointsToMakeAccessibleMain\n");

  execute_Mightability_Map_functions();

 #ifdef PR2_EXISTS_FOR_MA
 find_HRI_task_candidate_points(MAKE_OBJECT_ACCESSIBLE,"GREY_TAPE",PR2_MA,HUMAN1_MA,0,curr_resultant_candidate_points, CONSIDER_OBJECT_DIMENSION_FOR_CANDIDATE_PTS);
 #endif
 #ifdef JIDO_EXISTS_FOR_MA
 find_HRI_task_candidate_points(MAKE_OBJECT_ACCESSIBLE,"GREY_TAPE",JIDO_MA,HUMAN1_MA,0,curr_resultant_candidate_points, CONSIDER_OBJECT_DIMENSION_FOR_CANDIDATE_PTS);
 #endif

 int no_pts=0;
  for(int i=0;i<curr_resultant_candidate_points->no_points;i++)
  {
  if(strcasestr(envPt_MM->robot[curr_resultant_candidate_points->horizontal_surface_of[i]]->name,"HRP2TABLE"))
   {
  points_to_make_accessible->points[no_pts].x=curr_resultant_candidate_points->point[i].x;
  points_to_make_accessible->points[no_pts].y=curr_resultant_candidate_points->point[i].y;
  points_to_make_accessible->points[no_pts].z=curr_resultant_candidate_points->point[i].z;
  no_pts++; 
   }
  }
  points_to_make_accessible->no_points=no_pts;
 
  SHOW_CURRENT_TASK_CANDIDATE_POINTS=1;
  SHOW_WEIGHT_BY_COLOR_FOR_CANDIDATE_POINTS=1;

  printf(" Number of points for make accessible = %d\n",points_to_make_accessible->no_points);

  return END;
  
}

ACTIVITY_EVENT
mhpGetPointsToMakeAccessibleInter(STRUCT_POINTS_FOR_HRI_TASKS *points_to_make_accessible,int *report)
{
    return END;
}

ACTIVITY_EVENT
mhpGetPointsToMakeAccessibleEnd(STRUCT_POINTS_FOR_HRI_TASKS *points_to_make_accessible,int *report)
{
    printf(" Inside mhpGetPointsToMakeAccessibleEnd\n");
MY_FREE(curr_resultant_candidate_points, candidate_poins_for_task,1);
    return ETHER;
}


//For getting mean point for task
ACTIVITY_EVENT
mhpGetHriTasksMeanPointsStart( STRUCT_HRI_TASK_REQ *HRI_task_pro_IP, STRUCT_POINTS_FOR_HRI_TASKS *HRI_tasks_mean_points,int *report)
{
  printf(" Trying mhp_fetch_and_update_env(report)\n");
  if(!mhp_fetch_and_update_env(report))
  {
    printf(">>>>> HRI TASK INIT ERROR: Cannot fetch the current environment\n");
    *report=S_mhp_FAIL_TO_FETCH_ENV; 
    return END;
  }
  
  
  return EXEC;
}

ACTIVITY_EVENT
mhpGetHriTasksMeanPointsMain(STRUCT_HRI_TASK_REQ *HRI_task_pro_IP, STRUCT_POINTS_FOR_HRI_TASKS *HRI_tasks_mean_points,int *report)
{
  
  execute_Mightability_Map_functions();
/*
  HRI_task_desc task_to_validate;
  symbolic_HRI_task_desc symb_HRI_task_ip;

  strcpy(symb_HRI_task_ip.task_name,TASK_IP_FROM_SHARY->sh_task.task.task_name);
  strcpy(symb_HRI_task_ip.for_object,TASK_IP_FROM_SHARY->sh_task.task.for_object);
  strcpy(symb_HRI_task_ip.by_agent,TASK_IP_FROM_SHARY->sh_task.task.by_agent);
  strcpy(symb_HRI_task_ip.for_agent,TASK_IP_FROM_SHARY->sh_task.task.for_agent);

  ////printf(" >>>> Inside mhpPlan_HRI_TaskMain(), calling convert_symbolic_HRI_task_desc_into() with task name= %s, for object %s, by agent = %s, for agent= %s\n",symb_HRI_task_ip.task_name,symb_HRI_task_ip.for_object, symb_HRI_task_ip.by_agent,symb_HRI_task_ip.for_agent);

  if(convert_symbolic_HRI_task_desc_into(symb_HRI_task_ip, task_to_validate)<=0)
  {
   printf(" HRI_TASK ERROR : Could not convert the symbolic description of task to validate \n");
   return END;
  }
 
  int res=validate_HRI_task(task_to_validate, TASK_IP_FROM_SHARY->sh_task.task_plan_id);
  
  printf(" After validate_HRI_task \n");
  if(res<=0)
  {
   printf(" HRI_TASK ERROR: Fail to validate current task\n");
   *report=S_mhp_Plan_HRI_Task_for_SHARY_FAIL; 
   return END;
  }

   std::vector <SM_TRAJ> smTrajs;
   
   int sub_traj_st_index=0;
   int sub_traj_end_index=2;
   int indexTraj=TASK_IP_FROM_SHARY->traj_info[0].traj_index;
  ////curr_smTrajs.clear();
  

  int SM_traj_res=get_single_soft_motion_traj_for_SHARY(TASK_IP_FROM_SHARY->sh_task.task_plan_id, sub_traj_st_index, sub_traj_end_index, smTrajs[0] );
  if(SM_traj_res<=0)
  {
   printf(" HRI_TASK ERROR : Fail to get Soft Motion Trajectory with concatTrajectories \n");
   *report=S_mhp_Plan_HRI_Task_for_SHARY_FAIL; 
   return END;
  }

  smTrajs[0].convertToSM_TRAJ_STR(&(SDI_F->armTrajArray.traj[indexTraj]));

   sub_traj_st_index=3;
    sub_traj_end_index=4;
    indexTraj=TASK_IP_FROM_SHARY->traj_info[1].traj_index;

    SM_traj_res=get_single_soft_motion_traj_for_SHARY(TASK_IP_FROM_SHARY->sh_task.task_plan_id, sub_traj_st_index, sub_traj_end_index, smTrajs[1] );
  if(SM_traj_res<=0)
  {
   printf(" HRI_TASK ERROR : Fail to get Soft Motion Trajectory with concatTrajectories \n");
   *report=S_mhp_Plan_HRI_Task_for_SHARY_FAIL; 
   return END;
  }

  smTrajs[1].convertToSM_TRAJ_STR(&(SDI_F->armTrajArray.traj[indexTraj]));

   sub_traj_st_index=5;
    sub_traj_end_index=6;
    indexTraj=TASK_IP_FROM_SHARY->traj_info[2].traj_index;
  if(indexTraj<0)
  {
  return END;
  }

    SM_traj_res=get_single_soft_motion_traj_for_SHARY(TASK_IP_FROM_SHARY->sh_task.task_plan_id, sub_traj_st_index, sub_traj_end_index, smTrajs[2] );
  if(SM_traj_res<=0)
  {
   printf(" HRI_TASK ERROR : Fail to get Soft Motion Trajectory with concatTrajectories \n");
   *report=S_mhp_Plan_HRI_Task_for_SHARY_FAIL; 
   return END;
  }

  smTrajs[2].convertToSM_TRAJ_STR(&(SDI_F->armTrajArray.traj[indexTraj]));

  ////printf(" Number of soft motion trajectories = %d \n",curr_smTrajs.size());
*/
  return END;
}

ACTIVITY_EVENT
mhpGetHriTasksMeanPointsInter(STRUCT_HRI_TASK_REQ *HRI_task_pro_IP, STRUCT_POINTS_FOR_HRI_TASKS *HRI_tasks_mean_points,int *report)
{
    return END;
}

ACTIVITY_EVENT
mhpGetHriTasksMeanPointsEnd(STRUCT_HRI_TASK_REQ *HRI_task_pro_IP, STRUCT_POINTS_FOR_HRI_TASKS *HRI_tasks_mean_points,int *report)
{
    printf(" Inside mhpPlan_HRI_Task_for_SHARYEnd\n");
    return ETHER;
}

//Setting the HRI task planning mode
ACTIVITY_EVENT
mhpSetHRITaskPlanningModeStart(STRUCT_PLAN_MODE_FOR_HRI_TASKS *hri_task_planning_mode,int *report)
{
  
  return EXEC;
}

ACTIVITY_EVENT
mhpSetHRITaskPlanningModeMain(STRUCT_PLAN_MODE_FOR_HRI_TASKS *hri_task_planning_mode,int *report)
{
  HRI_TASK_PLAN_IN_CARTESIAN=hri_task_planning_mode->plan_in_cartesian;
  return END;
}

ACTIVITY_EVENT
mhpSetHRITaskPlanningModeInter(STRUCT_PLAN_MODE_FOR_HRI_TASKS *hri_task_planning_mode,int *report)
{
    return END;
}

ACTIVITY_EVENT
mhpSetHRITaskPlanningModeEnd(STRUCT_PLAN_MODE_FOR_HRI_TASKS *hri_task_planning_mode,int *report)
{
   
    return ETHER;
}

// To find the candidate points for a particular HRI task
ACTIVITY_EVENT
mhpGetCandidatePtsForHRITaskStart(STRUCT_HRI_TASK_REQ *HRI_task_IP_for_candi_points,int *report)
{
  printf(" Trying mhp_fetch_and_update_env(report)\n");
  if(!mhp_fetch_and_update_env(report))
  {
    printf(">>>>> HRI TASK INIT ERROR: Cannot fetch the current environment from SPARK, so there will risk of inconsistency during real execution so the candidate points for the task will NOT be calculated.\n");
    return END;
  }
  
  
  return EXEC;
}

ACTIVITY_EVENT
mhpGetCandidatePtsForHRITaskMain(STRUCT_HRI_TASK_REQ *HRI_task_IP_for_candi_points,int *report)
{
  printf(" >*>*>*> Inside mhpGetCandidatePtsForHRITaskMain() with task name= %s, for object %s, by agent = %s, for agent= %s\n",HRI_task_IP_for_candi_points->task.task_name,HRI_task_IP_for_candi_points->task.for_object, HRI_task_IP_for_candi_points->task.by_agent,HRI_task_IP_for_candi_points->task.for_agent);
  
  set_all_Mightability_Analyses_to_update();

  execute_Mightability_Map_functions();

  HRI_task_desc task_to_validate;
  symbolic_HRI_task_desc symb_HRI_task_ip;

  strcpy(symb_HRI_task_ip.task_name,HRI_task_IP_for_candi_points->task.task_name);
  strcpy(symb_HRI_task_ip.for_object,HRI_task_IP_for_candi_points->task.for_object);
  strcpy(symb_HRI_task_ip.by_agent,HRI_task_IP_for_candi_points->task.by_agent);
  strcpy(symb_HRI_task_ip.for_agent,HRI_task_IP_for_candi_points->task.for_agent);

  printf(" >>>> Inside mhpPlan_HRI_TaskMain(), calling convert_symbolic_HRI_task_desc_into() with task name= %s, for object %s, by agent = %s, for agent= %s\n",symb_HRI_task_ip.task_name,symb_HRI_task_ip.for_object, symb_HRI_task_ip.by_agent,symb_HRI_task_ip.for_agent);

  if(convert_symbolic_HRI_task_desc_into(symb_HRI_task_ip, task_to_validate)<=0)
  {
   printf(" HRI_TASK ERROR : Could not convert the symbolic description of task to validate \n");
   return END;
  }
  IS_PERFORMING_AGENT_MASTER=HRI_task_IP_for_candi_points->task.by_master;
  CONSIDER_OBJECT_DIMENSION_FOR_CANDIDATE_PTS=HRI_task_IP_for_candi_points->task.use_obj_dimension;

  get_candidate_points_for_HRI_task(task_to_validate, HRI_task_IP_for_candi_points->task.by_master, HRI_task_IP_for_candi_points->task.use_obj_dimension);
  
  printf(" After get_candidate_points_for_HRI_task \n");

 
  return END;
}

ACTIVITY_EVENT
mhpGetCandidatePtsForHRITaskInter(STRUCT_HRI_TASK_REQ *HRI_task_IP_for_candi_points,int *report)
{
    return END;
}

ACTIVITY_EVENT
mhpGetCandidatePtsForHRITaskEnd(STRUCT_HRI_TASK_REQ *HRI_task_IP_for_candi_points,int *report)
{
    printf(" Inside mhpGetCandidatePtsForHRITaskEnd\n");
    return ETHER;
}

//To set/unset of displaying the candidate points

 
ACTIVITY_EVENT
mhpShowCandidatePtsForHRITaskStart(int *report)
{
  
  return EXEC;
}

ACTIVITY_EVENT
mhpShowCandidatePtsForHRITaskMain(int *report)
{
 
  if(SHOW_CURRENT_TASK_CANDIDATE_POINTS==0)
  {
  
 SHOW_CURRENT_TASK_CANDIDATE_POINTS=1;
 SHOW_WEIGHT_BY_COLOR_FOR_CANDIDATE_POINTS=1;
  }
  else
  {
 SHOW_CURRENT_TASK_CANDIDATE_POINTS=0;
 SHOW_WEIGHT_BY_COLOR_FOR_CANDIDATE_POINTS=0;
  }

  return END;
}

ACTIVITY_EVENT
mhpShowCandidatePtsForHRITaskInter(int *report)
{
    return END;
}

ACTIVITY_EVENT
mhpShowCandidatePtsForHRITaskEnd(int *report)
{
    printf(" Inside mhpShowCandidatePtsForHRITaskEnd.\n");
    return ETHER;
}

// To set human's effort level for a particular HRI task
ACTIVITY_EVENT
mhpSetHumanEffortLevelForHRITaskStart(STRUCT_EFFORT_LEVEL_IP *hum_effort_level_for_task,int *report)
{
  
  return EXEC;
}

ACTIVITY_EVENT
mhpSetHumanEffortLevelForHRITaskMain(STRUCT_EFFORT_LEVEL_IP *hum_effort_level_for_task,int *report)
{
HRI_task_agent_effort_level desired_level;
 #ifdef PR2_EXISTS_FOR_MA
  desired_level.performing_agent=PR2_MA;
  desired_level.target_agent=HUMAN1_MA;
  
  desired_level.effort_for_agent=HUMAN1_MA;
  
  desired_level.maxi_reach_accept=(MA_transition_reach_effort_type)hum_effort_level_for_task->level_for_reachability;//MA_ARM_TORSO_EFFORT;//MA_WHOLE_BODY_CURR_POS_EFFORT_REACH;
  desired_level.maxi_vis_accept=(MA_transition_vis_effort_type)hum_effort_level_for_task->level_for_visibility;//MA_WHOLE_BODY_CURR_POS_EFFORT_VIS;//MA_HEAD_EFFORT;//MA_WHOLE_BODY_CURR_POS_EFFORT_VIS;
  
  int task_id=get_HRI_task_id_type_by_name(hum_effort_level_for_task->for_task, desired_level.task);

 if(task_id==-1)
 {
  printf(" mhp error: Unknown task %s\n",hum_effort_level_for_task->for_task); 
  return END;
 }

 printf(" >*>*>*> Inside mhpSetHumanEffortLevelForHRITaskMain() with task id=%d, task name= %s, reachability level=%d, visibility level=%d\n",desired_level.task, hum_effort_level_for_task->for_task, hum_effort_level_for_task->level_for_reachability, hum_effort_level_for_task->level_for_visibility);

 set_accepted_effort_level_for_HRI_task(desired_level);

 
  desired_level.performing_agent=HUMAN1_MA;
  desired_level.target_agent=PR2_MA;
  
  desired_level.effort_for_agent=HUMAN1_MA;

  set_accepted_effort_level_for_HRI_task(desired_level);

  printf(" After set_accepted_effort_level_for_HRI_task() \n");
#endif
 
  return END;
}

ACTIVITY_EVENT
mhpSetHumanEffortLevelForHRITaskInter(STRUCT_EFFORT_LEVEL_IP *hum_effort_level_for_task,int *report)
{
    return END;
}

ACTIVITY_EVENT
mhpSetHumanEffortLevelForHRITaskEnd(STRUCT_EFFORT_LEVEL_IP *hum_effort_level_for_task,int *report)
{
    printf(" Inside mhpSetHumanEffortLevelForHRITaskEnd\n");
    return ETHER;
}

//To get the support name to put the object by human for proactive planning
ACTIVITY_EVENT
mhpGetSupportNameProMkAccessStart(STRUCT_PROACTIVE_PLAN_RESULT *mk_access_result,int *report)
{
  
  return EXEC;
}

ACTIVITY_EVENT
mhpGetSupportNameProMkAccessMain(STRUCT_PROACTIVE_PLAN_RESULT *mk_access_result,int *report)
{

  strcpy(mk_access_result->support_name, SUPPORT_NAME_FOR_HUMAN_TO_PUT_OBJ);  
  printf(" Got Support name : %s\n",mk_access_result->support_name);
  return END;
}

ACTIVITY_EVENT
mhpGetSupportNameProMkAccessInter(STRUCT_PROACTIVE_PLAN_RESULT *mk_access_result,int *report)
{
    return END;
}

ACTIVITY_EVENT
mhpGetSupportNameProMkAccessEnd(STRUCT_PROACTIVE_PLAN_RESULT *mk_access_result,int *report)
{
    printf(" Inside mhpGetSupportNameProMkAccessEnd\n");
    return ETHER;
}
 

/* ------- END: Planning Requests for HRI Task ------- */
#endif

#if defined (JIDO)

static int trajID = 1;
static POSTER_ID lwrPosOnTrajPosterID = NULL;

static double tauSwitch= 5;
static int currTraj = 0;

/*------------------------------------------------------------------------
 * ArmPlanReplanLoop
 *
 * Description: 
 *
 * Reports:      OK
 *              S_mhp_CANNOT_FIND_POSTER
 *              S_mhp_CANNOT_READ_POSTER
 */

/* mhpArmPlanReplanLoopStart  -  codel START of ArmPlanReplanLoop
   Returns:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
mhpArmPlanReplanLoopStart(STRUCT_MHP_ARM_REPLAN_PARAMS *armReplanParams, int *report)
{
  if(!mhp_fetch_and_update_env(report)) {
    printf("Warning: Cannot fetch the current environment\n");
  }
  SDI_F->State.planning = TRUE; /* --- Planning Mode ---- */
  SDI_F->collisionState.state = GENMANIP_COLLISION_FREE;
  
  if (MANIP_PLANNER== NULL) {
    *report = S_mhp_NOT_INITIALIZED;
    return ETHER;
  }
  
  if(mhpGENMANIP_POSITION_STRPosterFind("lwrPosOnTraj", &lwrPosOnTrajPosterID)!= OK) {
    printf("mhpArmPlanTaskLoopStart cannot find the lwr PosOnTraj poster \n");
    return END;
  }

  if(armReplanParams->simulation == GEN_TRUE) {
    SDI_F->armTrajArray.traj[0].trajId = 0;
    SDI_F->armTrajArray.traj[0].timePreserved = tauSwitch;
    currTraj = 0;
    mhpArmTrajarmTrajectoryOuputPosterWrite( MHP_ARMTRAJ_POSTER_ID, &(SDI_F->armTrajArray.traj[0]));
  }
  return EXEC;
}

/* mhpArmPlanReplanLoopMain  -  codel EXEC of ArmPlanReplanLoop
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
mhpArmPlanReplanLoopMain(STRUCT_MHP_ARM_REPLAN_PARAMS *armReplanParams, int *report)
{
  
  SM_TRAJ smTraj;
  double t_rep = 4.0; // we allow a replanification during 4 second
  int lpIdDeform;
  GENMANIP_POSITION_STR lwrPosOnTrajData;
  
  if(!mhp_fetch_and_update_env(report)) {
    printf("Warning: Cannot fetch the current environment\n");
  }

  if(mhpGENMANIP_POSITION_STRPosterRead(lwrPosOnTrajPosterID, &lwrPosOnTrajData)!= OK) {
    printf("mhpArmPlanTaskLoopStart cannot READ the lwr PosOnTraj poster \n");
    return END;
  }
  printf(" tau %f \n", lwrPosOnTrajData.tau);
  
  if(armReplanParams->simulation == GEN_TRUE) {
    if((lwrPosOnTrajData.tau >  (tauSwitch-1))  && (currTraj < 1)) {
      printf("#################### MHP switching trajectory ###############\n");
      SDI_F->armTrajArray.traj[1].trajId = 1;
      SDI_F->armTrajArray.traj[1].timePreserved = tauSwitch;
      currTraj = 1;
      mhpArmTrajarmTrajectoryOuputPosterWrite( MHP_ARMTRAJ_POSTER_ID, &(SDI_F->armTrajArray.traj[1]));
      
    }
  } else {
    
    if(lwrPosOnTrajData.tau <=  3.0) {
      return EXEC;
    }


    if(p3d_getQSwitchIDFromMidCVS(lwrPosOnTrajData.tau, t_rep,  &lpIdDeform)!=0) {
      printf("INFO mhpArmPlanTaskLoopMain end of trajectory reached by lwr-genom\n");
      return END;
    }


    
#ifdef HRI_COSTSPACE

    ENV.setBool(Env::isCostSpace,true);
    MANIP_PLANNER->setReplanningMethod( replanning_Function );

    bool rightHand = true;
    Eigen::Vector3d WSPoint = dynamic_cast<HRICS::Workspace*>(HRICS_MotionPL)->computeOTPFromHandPose( rightHand );
    
    p3d_vector3 otp;
    otp[0] = WSPoint[0];
    otp[1] = WSPoint[1];
    otp[2] = WSPoint[2];

    if( MANIP_PLANNER->armReplan(otp,lpIdDeform,smTraj) != MANIPULATION_TASK_OK) {
      trajID++;
      smTraj.convertToSM_TRAJ_STR(&(SDI_F->armTrajArray.traj[2]));
      SDI_F->armTrajArray.traj[2].trajId = trajID;
      mhpArmTrajarmTrajectoryOuputPosterWrite( MHP_ARMTRAJ_POSTER_ID, &(SDI_F->armTrajArray.traj[2]));
    }
#endif
  }
  return EXEC;
}

/* mhpArmPlanReplanLoopEnd  -  codel END of ArmPlanReplanLoop
   Returns:  END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
mhpArmPlanReplanLoopEnd(STRUCT_MHP_ARM_REPLAN_PARAMS *armReplanParams, int *report)
{
  deactivateCcCntrts(MHP_ROBOT, -1);
  SDI_F->State.planning = FALSE;
  return ETHER;
}
#endif

#if defined (JIDO) || defined (PR2)
/*------------------------------------------------------------------------
 * LoadSoftMotionTraj
 *
 * Description: 
 *
 * Reports:      OK
 *              S_mhp_NOT_INITIALIZED
 *              S_mhp_CANNOT_LOAD_FILE
 *              S_mhp_WRONG_ID
 */

/* mhpLoadSoftMotionTrajMain  -  codel EXEC of LoadSoftMotionTraj
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
mhpLoadSoftMotionTrajMain(STRUCT_MHP_LOADTRAJ_PARAMS *loadTrajParams, int *report)
{
  
  SM_TRAJ myTraj;

  int id = loadTrajParams->idToStore;

  if(id<0 || id > MHP_ARM_NB_MAX_TRAJ) {
    *report = S_mhp_WRONG_ID;
    return ETHER;
  }

  if(myTraj.load(loadTrajParams->fileName.name, NULL)!=0) {
    *report = S_mhp_CANNOT_LOAD_FILE;
    return ETHER;
  }

  myTraj.convertToSM_TRAJ_STR(&(SDI_F->armTrajArray.traj[id]));

  SDI_F->armTrajArray.traj[id].trajId = id;

  return ETHER;
}


#endif
